@@ -8,39 +8,37 @@ import tempfile
 import time
 
 
-try:
-    from stat import filemode as _filemode  # PY 3.3
-except ImportError:
-    from tarfile import filemode as _filemode
 try:
     import grp
     import pwd
 except ImportError:
     pwd = grp = None
-try:
-    from os import scandir  # py 3.5
-except ImportError:
-    try:
-        from scandir import scandir  # requires "pip install scandir"
-    except ImportError:
-        scandir = None
-
-from ._compat import PY3
-from ._compat import u
-from ._compat import unicode
 
 
-__all__ = ['FilesystemError', 'AbstractedFS']
+__all__ = ['AbstractedFS', 'FilesystemError']
 
 
-_months_map = {1: 'Jan', 2: 'Feb', 3: 'Mar', 4: 'Apr', 5: 'May', 6: 'Jun',
-               7: 'Jul', 8: 'Aug', 9: 'Sep', 10: 'Oct', 11: 'Nov', 12: 'Dec'}
+_months_map = {
+    1: 'Jan',
+    2: 'Feb',
+    3: 'Mar',
+    4: 'Apr',
+    5: 'May',
+    6: 'Jun',
+    7: 'Jul',
+    8: 'Aug',
+    9: 'Sep',
+    10: 'Oct',
+    11: 'Nov',
+    12: 'Dec',
+}
 
 
 def _memoize(fun):
     """A simple memoize decorator for functions supporting (hashable)
     positional arguments.
     """
+
     def wrapper(*args, **kwargs):
         key = (args, frozenset(sorted(kwargs.items())))
         try:
@@ -57,6 +55,7 @@ def _memoize(fun):
 # --- custom exceptions
 # ===================================================================
 
+
 class FilesystemError(Exception):
     """Custom class for filesystem-related exceptions.
     You can raise this from an AbstractedFS subclass in order to
@@ -68,7 +67,8 @@ class FilesystemError(Exception):
 # --- base class
 # ===================================================================
 
-class AbstractedFS(object):
+
+class AbstractedFS:
     """A class used to interact with the file system, providing a
     cross-platform interface compatible with both Windows and
     UNIX style filesystems where all paths use "/" separator.
@@ -89,16 +89,15 @@ class AbstractedFS(object):
 
     def __init__(self, root, cmd_channel):
         """
-         - (str) root: the user "real" home directory (e.g. '/home/user')
-         - (instance) cmd_channel: the FTPHandler class instance
+        - (str) root: the user "real" home directory (e.g. '/home/user')
+        - (instance) cmd_channel: the FTPHandler class instance.
         """
-        assert isinstance(root, unicode)
         # Set initial current working directory.
         # By default initial cwd is set to "/" to emulate a chroot jail.
         # If a different behavior is desired (e.g. initial cwd = root,
         # to reflect the real filesystem) users overriding this class
         # are responsible to set _cwd attribute as necessary.
-        self._cwd = u('/')
+        self._cwd = '/'
         self._root = root
         self.cmd_channel = cmd_channel
 
@@ -114,12 +113,10 @@ class AbstractedFS(object):
 
     @root.setter
     def root(self, path):
-        assert isinstance(path, unicode), path
         self._root = path
 
     @cwd.setter
     def cwd(self, path):
-        assert isinstance(path, unicode), path
         self._cwd = path
 
     # --- Pathname / conversion utilities
@@ -135,7 +132,6 @@ class AbstractedFS(object):
         Note: directory separators are system independent ("/").
         Pathname returned is always absolutized.
         """
-        assert isinstance(ftppath, unicode), ftppath
         if os.path.isabs(ftppath):
             p = os.path.normpath(ftppath)
         else:
@@ -153,7 +149,7 @@ class AbstractedFS(object):
         # that self.cwd is not absolute, return "/" as a safety measure.
         # This is for extra protection, maybe not really necessary.
         if not os.path.isabs(p):
-            p = u("/")
+            p = "/"
         return p
 
     def ftp2fs(self, ftppath):
@@ -167,7 +163,6 @@ class AbstractedFS(object):
 
         Note: directory separators are system dependent.
         """
-        assert isinstance(ftppath, unicode), ftppath
         # as far as I know, it should always be path traversal safe...
         if os.path.normpath(self.root) == os.sep:
             return os.path.normpath(self.ftpnorm(ftppath))
@@ -190,15 +185,14 @@ class AbstractedFS(object):
         On invalid pathnames escaping from user's root directory
         (e.g. "/home" when root is "/home/user") always return "/".
         """
-        assert isinstance(fspath, unicode), fspath
         if os.path.isabs(fspath):
             p = os.path.normpath(fspath)
         else:
             p = os.path.normpath(os.path.join(self.root, fspath))
         if not self.validpath(p):
-            return u('/')
+            return '/'
         p = p.replace(os.sep, "/")
-        p = p[len(self.root):]
+        p = p[len(self.root) :]
         if not p.startswith('/'):
             p = '/' + p
         return p
@@ -213,22 +207,18 @@ class AbstractedFS(object):
         Pathnames escaping from user's root directory are considered
         not valid.
         """
-        assert isinstance(path, unicode), path
         root = self.realpath(self.root)
         path = self.realpath(path)
         if not root.endswith(os.sep):
-            root = root + os.sep
+            root += os.sep
         if not path.endswith(os.sep):
-            path = path + os.sep
-        if path[0:len(root)] == root:
-            return True
-        return False
+            path += os.sep
+        return path[0 : len(root)] == root
 
     # --- Wrapper methods around open() and tempfile.mkstemp
 
     def open(self, filename, mode):
         """Open a file returning its handler."""
-        assert isinstance(filename, unicode), filename
         return open(filename, mode)
 
     def mkstemp(self, suffix='', prefix='', dir=None, mode='wb'):
@@ -236,6 +226,7 @@ class AbstractedFS(object):
         name.  Unlike mkstemp it returns an object with a file-like
         interface.
         """
+
         class FileWrapper:
 
             def __init__(self, fd, name):
@@ -259,103 +250,87 @@ class AbstractedFS(object):
         it is vital that `cwd` attribute gets set.
         """
         # note: process cwd will be reset by the caller
-        assert isinstance(path, unicode), path
         os.chdir(path)
         self.cwd = self.fs2ftp(path)
 
     def mkdir(self, path):
         """Create the specified directory."""
-        assert isinstance(path, unicode), path
         os.mkdir(path)
 
     def listdir(self, path):
         """List the content of a directory."""
-        assert isinstance(path, unicode), path
         return os.listdir(path)
 
     def listdirinfo(self, path):
         """List the content of a directory."""
-        assert isinstance(path, unicode), path
         return os.listdir(path)
 
     def rmdir(self, path):
         """Remove the specified directory."""
-        assert isinstance(path, unicode), path
         os.rmdir(path)
 
     def remove(self, path):
         """Remove the specified file."""
-        assert isinstance(path, unicode), path
         os.remove(path)
 
     def rename(self, src, dst):
         """Rename the specified src file to the dst filename."""
-        assert isinstance(src, unicode), src
-        assert isinstance(dst, unicode), dst
         os.rename(src, dst)
 
     def chmod(self, path, mode):
         """Change file/directory mode."""
-        assert isinstance(path, unicode), path
         if not hasattr(os, 'chmod'):
             raise NotImplementedError
         os.chmod(path, mode)
 
     def stat(self, path):
         """Perform a stat() system call on the given path."""
-        # on python 2 we might also get bytes from os.lisdir()
-        # assert isinstance(path, unicode), path
         return os.stat(path)
 
     def utime(self, path, timeval):
-        """Perform a utime() call on the given path"""
+        """Perform a utime() call on the given path."""
         # utime expects a int/float (atime, mtime) in seconds
         # thus, setting both access and modify time to timeval
         return os.utime(path, (timeval, timeval))
 
     if hasattr(os, 'lstat'):
+
         def lstat(self, path):
             """Like stat but does not follow symbolic links."""
-            # on python 2 we might also get bytes from os.lisdir()
-            # assert isinstance(path, unicode), path
             return os.lstat(path)
+
     else:
         lstat = stat
 
     if hasattr(os, 'readlink'):
+
         def readlink(self, path):
             """Return a string representing the path to which a
             symbolic link points.
             """
-            assert isinstance(path, unicode), path
             return os.readlink(path)
 
     # --- Wrapper methods around os.path.* calls
 
     def isfile(self, path):
         """Return True if path is a file."""
-        assert isinstance(path, unicode), path
         return os.path.isfile(path)
 
     def islink(self, path):
         """Return True if path is a symbolic link."""
-        assert isinstance(path, unicode), path
         return os.path.islink(path)
 
     def isdir(self, path):
         """Return True if path is a directory."""
-        assert isinstance(path, unicode), path
         return os.path.isdir(path)
 
     def getsize(self, path):
         """Return the size of the specified file in bytes."""
-        assert isinstance(path, unicode), path
         return os.path.getsize(path)
 
     def getmtime(self, path):
         """Return the last modified time as a number of seconds since
         the epoch."""
-        assert isinstance(path, unicode), path
         return os.path.getmtime(path)
 
     def realpath(self, path):
@@ -363,17 +338,16 @@ class AbstractedFS(object):
         symbolic links encountered in the path (if they are
         supported by the operating system).
         """
-        assert isinstance(path, unicode), path
         return os.path.realpath(path)
 
     def lexists(self, path):
         """Return True if path refers to an existing path, including
         a broken or circular symbolic link.
         """
-        assert isinstance(path, unicode), path
         return os.path.lexists(path)
 
     if pwd is not None:
+
         def get_user_by_uid(self, uid):
             """Return the username associated with user id.
             If this can't be determined return raw uid instead.
@@ -383,11 +357,14 @@ class AbstractedFS(object):
                 return pwd.getpwuid(uid).pw_name
             except KeyError:
                 return uid
+
     else:
+
         def get_user_by_uid(self, uid):
             return "owner"
 
     if grp is not None:
+
         def get_group_by_gid(self, gid):
             """Return the group name associated with group id.
             If this can't be determined return raw gid instead.
@@ -397,7 +374,9 @@ class AbstractedFS(object):
                 return grp.getgrgid(gid).gr_name
             except KeyError:
                 return gid
+
     else:
+
         def get_group_by_gid(self, gid):
             return "group"
 
@@ -424,6 +403,7 @@ class AbstractedFS(object):
         drwxrwxrwx   1 owner   group          0 Aug 31 18:50 e-books
         -rw-rw-rw-   1 owner   group        380 Sep 02  3:40 module.py
         """
+
         @_memoize
         def get_user_by_uid(uid):
             return self.get_user_by_uid(uid)
@@ -432,7 +412,6 @@ class AbstractedFS(object):
         def get_group_by_gid(gid):
             return self.get_group_by_gid(gid)
 
-        assert isinstance(basedir, unicode), basedir
         if self.cmd_channel.use_gmt_times:
             timefunc = time.gmtime
         else:
@@ -441,20 +420,7 @@ class AbstractedFS(object):
         readlink = getattr(self, 'readlink', None)
         now = time.time()
         for basename in listing:
-            if not PY3:
-                try:
-                    file = os.path.join(basedir, basename)
-                except UnicodeDecodeError:
-                    # (Python 2 only) might happen on filesystem not
-                    # supporting UTF8 meaning os.listdir() returned a list
-                    # of mixed bytes and unicode strings:
-                    # http://goo.gl/6DLHD
-                    # http://bugs.python.org/issue683592
-                    file = os.path.join(bytes(basedir), bytes(basename))
-                    if not isinstance(basename, unicode):
-                        basename = unicode(basename, 'utf8', 'ignore')
-            else:
-                file = os.path.join(basedir, basename)
+            file = os.path.join(basedir, basename)
             try:
                 st = self.lstat(file)
             except (OSError, FilesystemError):
@@ -462,7 +428,7 @@ class AbstractedFS(object):
                     continue
                 raise
 
-            perms = _filemode(st.st_mode)  # permissions
+            perms = stat.filemode(st.st_mode)  # permissions
             nlinks = st.st_nlink  # number of links to inode
             if not nlinks:  # non-posix system, let's use a bogus value
                 nlinks = 1
@@ -473,20 +439,21 @@ class AbstractedFS(object):
             # if modification time > 6 months shows "month year"
             # else "month hh:mm";  this matches proftpd format, see:
             # https://github.com/giampaolo/pyftpdlib/issues/187
-            if (now - st.st_mtime) > SIX_MONTHS:
-                fmtstr = "%d  %Y"
-            else:
-                fmtstr = "%d %H:%M"
+            fmtstr = '%d  %Y' if now - st.st_mtime > SIX_MONTHS else '%d %H:%M'
             try:
-                mtimestr = "%s %s" % (_months_map[mtime.tm_mon],
-                                      time.strftime(fmtstr, mtime))
+                mtimestr = "%s %s" % (  # noqa: UP031
+                    _months_map[mtime.tm_mon],
+                    time.strftime(fmtstr, mtime),
+                )
             except ValueError:
                 # It could be raised if last mtime happens to be too
                 # old (prior to year 1900) in which case we return
                 # the current time as last mtime.
                 mtime = timefunc()
-                mtimestr = "%s %s" % (_months_map[mtime.tm_mon],
-                                      time.strftime("%d %H:%M", mtime))
+                mtimestr = "%s %s" % (  # noqa: UP031
+                    _months_map[mtime.tm_mon],
+                    time.strftime("%d %H:%M", mtime),
+                )
 
             # same as stat.S_ISLNK(st.st_mode) but slighlty faster
             islink = (st.st_mode & 61440) == stat.S_IFLNK
@@ -501,8 +468,17 @@ class AbstractedFS(object):
 
             # formatting is matched with proftpd ls output
             line = "%s %3s %-8s %-8s %8s %s %s\r\n" % (
-                perms, nlinks, uname, gname, size, mtimestr, basename)
-            yield line.encode('utf8', self.cmd_channel.unicode_errors)
+                perms,
+                nlinks,
+                uname,
+                gname,
+                size,
+                mtimestr,
+                basename,
+            )
+            yield line.encode(
+                self.cmd_channel.encoding, self.cmd_channel.unicode_errors
+            )
 
     def format_mlsx(self, basedir, listing, perms, facts, ignore_err=True):
         """Return an iterator object that yields the entries of a given
@@ -530,7 +506,6 @@ class AbstractedFS(object):
         type=dir;size=0;perm=el;modify=20071127230206;unique=801e33; ebooks
         type=file;size=211;perm=r;modify=20071103093626;unique=192; module.py
         """
-        assert isinstance(basedir, unicode), basedir
         if self.cmd_channel.use_gmt_times:
             timefunc = time.gmtime
         else:
@@ -551,21 +526,8 @@ class AbstractedFS(object):
         show_gid = 'unix.gid' in facts
         show_unique = 'unique' in facts
         for basename in listing:
-            retfacts = dict()
-            if not PY3:
-                try:
-                    file = os.path.join(basedir, basename)
-                except UnicodeDecodeError:
-                    # (Python 2 only) might happen on filesystem not
-                    # supporting UTF8 meaning os.listdir() returned a list
-                    # of mixed bytes and unicode strings:
-                    # http://goo.gl/6DLHD
-                    # http://bugs.python.org/issue683592
-                    file = os.path.join(bytes(basedir), bytes(basename))
-                    if not isinstance(basename, unicode):
-                        basename = unicode(basename, 'utf8', 'ignore')
-            else:
-                file = os.path.join(basedir, basename)
+            retfacts = {}
+            file = os.path.join(basedir, basename)
             # in order to properly implement 'unique' fact (RFC-3659,
             # chapter 7.5.2) we are supposed to follow symlinks, hence
             # use os.stat() instead of os.lstat()
@@ -598,8 +560,9 @@ class AbstractedFS(object):
             # last modification time
             if show_modify:
                 try:
-                    retfacts['modify'] = time.strftime("%Y%m%d%H%M%S",
-                                                       timefunc(st.st_mtime))
+                    retfacts['modify'] = time.strftime(
+                        "%Y%m%d%H%M%S", timefunc(st.st_mtime)
+                    )
                 # it could be raised if last mtime happens to be too old
                 # (prior to year 1900)
                 except ValueError:
@@ -607,8 +570,9 @@ class AbstractedFS(object):
             if show_create:
                 # on Windows we can provide also the creation time
                 try:
-                    retfacts['create'] = time.strftime("%Y%m%d%H%M%S",
-                                                       timefunc(st.st_ctime))
+                    retfacts['create'] = time.strftime(
+                        "%Y%m%d%H%M%S", timefunc(st.st_ctime)
+                    )
                 except ValueError:
                     pass
             # UNIX only
@@ -628,13 +592,16 @@ class AbstractedFS(object):
             # platforms should use some platform-specific method (e.g.
             # on Windows NTFS filesystems MTF records could be used).
             if show_unique:
-                retfacts['unique'] = "%xg%x" % (st.st_dev, st.st_ino)
+                retfacts['unique'] = f"{st.st_dev:x}g{st.st_ino:x}"
 
             # facts can be in any order but we sort them by name
-            factstring = "".join(["%s=%s;" % (x, retfacts[x])
-                                  for x in sorted(retfacts.keys())])
-            line = "%s %s\r\n" % (factstring, basename)
-            yield line.encode('utf8', self.cmd_channel.unicode_errors)
+            factstring = "".join(
+                [f"{x}={retfacts[x]};" for x in sorted(retfacts.keys())]
+            )
+            line = f"{factstring} {basename}\r\n"
+            yield line.encode(
+                self.cmd_channel.encoding, self.cmd_channel.unicode_errors
+            )
 
 
 # ===================================================================
@@ -642,7 +609,7 @@ class AbstractedFS(object):
 # ===================================================================
 
 if os.name == 'posix':
-    __all__.append('UnixFilesystem')
+    __all__ += ['UnixFilesystem']
 
     class UnixFilesystem(AbstractedFS):
         """Represents the real UNIX filesystem.
